package com.zy.bms.common.aspect;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zy.bms.common.ServerResponse;
import com.zy.bms.common.enums.ResponseCode;
import com.zy.bms.entity.User;
import com.zy.bms.service.IUserService;
import com.zy.bms.utils.AesUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * 微信用户是否是管理员权限拦截
 */
@Slf4j
@Aspect
@Component
public class AdminAuthorWxAop {

    @Resource
    private IUserService userService;


    @Around(value = "@annotation(com.zy.bms.common.annotation.AdminAuthorWx)")
    public Object doAround(ProceedingJoinPoint joinPoint) {
        try {
            //获取request对象
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String userId = AesUtil.decrypt(request.getHeader("app_user"));
            User user = userService.getOne(new QueryWrapper<User>().eq("id", userId));
            if (user != null && user.getAdmin() == 1) return joinPoint.proceed();
        } catch (Throwable throwable) {
            log.error("微信用户判断管理员权限AOP错误", throwable);
            throwable.printStackTrace();
        }
        return ServerResponse.custom(ResponseCode.AUTHOR);
    }
}